;*************************************************************************
;*									 *
;*				ATtiny45 Bootloader			 *
;*									 *
;*			Author: Peter Dannegger				 *
;*									 *
;*************************************************************************
.nolist
.include "tn45def.inc"
.include "tinyload.h"
.include "macros.inc"
.listmac
.list
;
	.org	0x0000
	rjmp	init

	.org	BOOTSTART
init:
	cli
	LDIW	zh,zl, RAMEND
	OUTW	SPH, SPL, zh, zl
	clr	zerol
	clr	zeroh
	clr	Flags
	sbi	SRX_PORT, SRX
	sbi	STX_PORT, STX
	sbi	STX_DDR, STX
;-------------------------------------------------------------------------
.include "abaud.inc"
;-------------------------------------------------------------------------
;				Check password
;-------------------------------------------------------------------------
;	sbi	ddrb, 2
checkpwd:
	ldi	zl, low (2*Password)
	ldi	zh, high(2*Password)
	rjmp	_cpw3
_cpw2:
	rcall	getchar
	cpse	r0, a0
	rjmp	checkpwd
_cpw3:
	lpm	r0, z+
	tst	r0
	brne	_cpw2
;	sbi	ddrb, 3
;-------------------------------------------------------------------------
;				Receive commands
;-------------------------------------------------------------------------
;A5,00 get bootloader revision
;A5,01 get buffer size
;A5,02 get target signature
;A5,03 get user flash size
;A5,04 program flash
;A5,05 start application
;-------------------------------------------------------------------------
mainloop:
	ldi	a0, SUCCESS
	sbrc	Flags, Ferror
	ldi	a0, FAIL
	cbr	Flags, 1<<Ferror
	rcall	putchar
_cex1:
	rcall	getchar
	cpi	a0, COMMAND
	brne	_cex1
	rcall	getchar
	cpi	a0, 4
	breq	program			; 4
	brcs	sendmessage		; 0 ... 3
	cpi	a0, 6
	brcc	_cex2			; >5
_timeout:
	out	DDRB, zerol
	out	PORTB, zerol
	rjmp	BootStart-1		; run user application
_cex2:
	sbr	flags, 1<<Ferror
	rjmp	mainloop
;-------------------------------------------------------------------------
;				Send Messages
;-------------------------------------------------------------------------
;input: a0 = Message index
;
SendMessage:
	mov	a2, a0
	ldi	zl, low (2*Messages)
	ldi	zh, high(2*Messages)
	ldi	a0, ANSWER
	rcall	putchar
	ldi	a0, 0
_sme1:
	add	zl, a0
	adc	zh, zeroh
	lpm	a0, z+
	subi	a2, 1
	brcc	_sme1
	mov	a2, a0
_sme2:
	rcall	putchar
	lpm	a0, z+
	subi	a2, 1
	brcc	_sme2
	rjmp	mainloop
;-------------------------------------------------------------------------
.include "program.inc"
	rjmp	MainLoop
;-------------------------------------------------------------------------
.include "uart.inc"
;-------------------------------------------------------------------------
Password:
	.db	"Peda", 0, 0
;
Messages:
	.db \
	2, 0x01, 0x00, \
	2, high(2*PageSize), low(2*PageSize), \
	3, SIGNATURE_000, SIGNATURE_001, SIGNATURE_002, \
	3, byte3(2*BootStart-2), byte2(2*BootStart-2), byte1(2*BootStart-2)
;-------------------------------------------------------------------------
